"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@babel/core");
var getTypeName_1 = __importDefault(require("./getTypeName"));
function isReactTypeMatch(name, type, reactImportedName) {
    return name === type || name === "React." + type || name === reactImportedName + "." + type;
}
function wrapIsRequired(propType, optional) {
    return optional ? propType : core_1.types.memberExpression(propType, core_1.types.identifier('isRequired'));
}
function createCall(value, args, propTypesImportedName) {
    return core_1.types.callExpression(createMember(value, propTypesImportedName), args);
}
function createMember(value, propTypesImportedName) {
    return core_1.types.memberExpression(core_1.types.identifier(propTypesImportedName), value);
}
function convert(type, options) {
    var reactImportedName = options.reactImportedName, propTypesImportedName = options.propTypesImportedName;
    // Remove wrapping parens
    if (core_1.types.isTSParenthesizedType(type)) {
        type = type.typeAnnotation;
    }
    // string -> PropTypes.string
    if (core_1.types.isTSStringKeyword(type)) {
        return createMember(core_1.types.identifier('string'), propTypesImportedName);
        // number -> PropTypes.number
    }
    else if (core_1.types.isTSNumberKeyword(type)) {
        return createMember(core_1.types.identifier('number'), propTypesImportedName);
        // boolean -> PropTypes.bool
    }
    else if (core_1.types.isTSBooleanKeyword(type)) {
        return createMember(core_1.types.identifier('bool'), propTypesImportedName);
        // symbol -> PropTypes.symbol
    }
    else if (core_1.types.isTSSymbolKeyword(type)) {
        return createMember(core_1.types.identifier('symbol'), propTypesImportedName);
        // object -> PropTypes.object
    }
    else if (core_1.types.isTSObjectKeyword(type)) {
        return createMember(core_1.types.identifier('object'), propTypesImportedName);
        // (() => void) -> PropTypes.func
    }
    else if (core_1.types.isTSFunctionType(type)) {
        return createMember(core_1.types.identifier('func'), propTypesImportedName);
        // React.ReactNode -> PropTypes.node
        // React.ReactElement -> PropTypes.element
        // React.MouseEvent -> PropTypes.object
        // React.MouseEventHandler -> PropTypes.func
        // React.Ref -> PropTypes.oneOfType()
        // JSX.Element -> PropTypes.element
        // CustomType -> PropTypes.any TODO
    }
    else if (core_1.types.isTSTypeReference(type)) {
        var name_1 = getTypeName_1.default(type.typeName);
        // node
        if (isReactTypeMatch(name_1, 'ReactText', reactImportedName) ||
            isReactTypeMatch(name_1, 'ReactNode', reactImportedName) ||
            isReactTypeMatch(name_1, 'ReactType', reactImportedName) ||
            isReactTypeMatch(name_1, 'ComponentType', reactImportedName) ||
            isReactTypeMatch(name_1, 'ComponentClass', reactImportedName) ||
            isReactTypeMatch(name_1, 'StatelessComponent', reactImportedName)) {
            return createMember(core_1.types.identifier('node'), propTypesImportedName);
            // element
        }
        else if (isReactTypeMatch(name_1, 'Element', 'JSX') ||
            isReactTypeMatch(name_1, 'ReactElement', reactImportedName) ||
            isReactTypeMatch(name_1, 'SFCElement', reactImportedName)) {
            return createMember(core_1.types.identifier('element'), propTypesImportedName);
            // oneOfType
        }
        else if (isReactTypeMatch(name_1, 'Ref', reactImportedName)) {
            return createCall(core_1.types.identifier('oneOfType'), [
                core_1.types.arrayExpression([
                    createMember(core_1.types.identifier('string'), propTypesImportedName),
                    createMember(core_1.types.identifier('func'), propTypesImportedName),
                    createMember(core_1.types.identifier('object'), propTypesImportedName),
                ]),
            ], propTypesImportedName);
            // func
        }
        else if (name_1.endsWith('Handler')) {
            return createMember(core_1.types.identifier('func'), propTypesImportedName);
            // object
        }
        else if (name_1.endsWith('Event')) {
            return createMember(core_1.types.identifier('object'), propTypesImportedName);
            // any (we need to support all these in case of unions)
        }
        else {
            return createMember(core_1.types.identifier('any'), propTypesImportedName);
        }
        // [] -> PropTypes.arrayOf(), PropTypes.array
    }
    else if (core_1.types.isTSArrayType(type)) {
        var args = convertArray([type.elementType], options);
        return args.length > 0
            ? createCall(core_1.types.identifier('arrayOf'), args, propTypesImportedName)
            : createMember(core_1.types.identifier('array'), propTypesImportedName);
        // {} -> PropTypes.object
        // { [key: string]: string } -> PropTypes.objectOf(PropTypes.string)
        // { foo: string } -> PropTypes.shape({ foo: PropTypes.string })
    }
    else if (core_1.types.isTSTypeLiteral(type)) {
        // object
        if (type.members.length === 0) {
            return createMember(core_1.types.identifier('object'), propTypesImportedName);
            // objectOf
        }
        else if (type.members.length === 1 && core_1.types.isTSIndexSignature(type.members[0])) {
            var index = type.members[0];
            if (index.typeAnnotation && index.typeAnnotation.typeAnnotation) {
                var result = convert(index.typeAnnotation.typeAnnotation, options);
                if (result) {
                    return createCall(core_1.types.identifier('objectOf'), [result], propTypesImportedName);
                }
            }
            // shape
        }
        else {
            return createCall(core_1.types.identifier('shape'), [
                core_1.types.objectExpression(convertListToProps(type.members.filter(function (member) {
                    return core_1.types.isTSPropertySignature(member);
                }), options)),
            ], propTypesImportedName);
        }
        // string | number -> PropTypes.oneOfType([PropTypes.string, PropTypes.number])
        // 'foo' | 'bar' -> PropTypes.oneOf(['foo', 'bar'])
    }
    else if (core_1.types.isTSUnionType(type) || core_1.types.isTSIntersectionType(type)) {
        var isAllLiterals = type.types.every(function (param) { return core_1.types.isTSLiteralType(param); });
        var label = void 0;
        var args = void 0;
        if (isAllLiterals) {
            args = type.types.map(function (param) { return param.literal; });
            label = core_1.types.identifier('oneOf');
        }
        else {
            args = convertArray(type.types, options);
            label = core_1.types.identifier('oneOfType');
        }
        if (label && args.length > 0) {
            return createCall(label, [core_1.types.arrayExpression(args)], propTypesImportedName);
        }
    }
    return null;
}
function convertArray(types, options) {
    var propTypes = [];
    types.forEach(function (type) {
        var prop = convert(type, options);
        if (prop) {
            propTypes.push(prop);
        }
    });
    return propTypes;
}
function convertListToProps(properties, options) {
    var propTypes = [];
    properties.forEach(function (property) {
        if (!property.typeAnnotation) {
            return;
        }
        var propType = convert(property.typeAnnotation.typeAnnotation, options);
        if (propType) {
            propTypes.push(core_1.types.objectProperty(property.key, wrapIsRequired(propType, property.optional)));
        }
    });
    return propTypes;
}
function convertToPropTypes(types, typeNames, options) {
    var properties = [];
    typeNames.forEach(function (typeName) {
        if (types[typeName]) {
            properties.push.apply(properties, convertListToProps(types[typeName], options));
        }
    });
    return properties;
}
exports.default = convertToPropTypes;
